/////////////////////////////////////////////////////////////////////////////
//
//		Name:			CConfigFile
//
//		Written by:		Martin Lewis
//
//		Date:			October 1996
//
//						rewritten Feb 2011 to use wstring rather than CString.
//
//						A very quick and dirty class for loading in
//						various types of application parameters from
//						a text file of some sort. ( eg. a .ini file )
//
/////////////////////////////////////////////////////////////////////////////

#include "configfile.h"


/////////////////////////////////////////////////////////////////////////////
// CGetDetails


CConfigFile::CConfigFile()
{
	m_iNumFileAccessRetries = 5;

	TextBuffer = L"";
}

CConfigFile::~CConfigFile()
{
}



///////////////////////////////////////////////////////////////////////////////////
//
// Search for the IniFileName provided. if found open it and load it into memory.


/////////////////////////////////////////////////////////////////////////////

bool CConfigFile::LoadFile( wstring Filename )
{
	int iRetries = m_iNumFileAccessRetries;

	while( RawLoadFile( Filename ) == FALSE && iRetries > 0 ) 
	{
		Sleep(100);
		iRetries--;
	}

	if ( iRetries <= 0 ) return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////


bool CConfigFile::RawLoadFile( wstring Filename )
{
	TextBuffer = L"";

	HANDLE hFile;
	
	wstring message;
	wstring Text;

	m_Filename = Filename;

	//-----------------------------------------------------

	hFile = CreateFile( Filename.c_str(),						
						GENERIC_READ,					// read/write
						FILE_SHARE_READ | FILE_SHARE_WRITE,	// share both
						NULL,							// no security
						OPEN_EXISTING,					// always work
						NULL,							// no overlap
						NULL );							// no template


	//-----------------------------------------------------

	if ( hFile == INVALID_HANDLE_VALUE )
	{
		return FALSE;
	}

	//-----------------------------------------------------
	
	DWORD FileSize = GetFileSize( hFile, NULL );

	if ( FileSize == 0xFFFFFFFF || FileSize > 50000 )		// if error or just too big to be correct.
	{
		CloseHandle( hFile );
		return FALSE;
	}

	char* pBuffer = new char[FileSize+2];

	if ( pBuffer == NULL ) 
	{
		CloseHandle( hFile );
		return FALSE;
	}

	//-----------------------------------------------------

	pBuffer[0] = 0;

	DWORD num_read;

	BOOL ReadStatus = ReadFile( hFile, pBuffer, FileSize+2, &num_read, NULL );

	if ( ReadStatus == FALSE ) num_read = 0;

	pBuffer[num_read] = 0;			// terminate the buffer correctly

	int nLen = MultiByteToWideChar(CP_ACP, 0, pBuffer, -1, NULL, NULL);
	
	wchar_t* lpwStr = new wchar_t[ nLen + 2 ];

	MultiByteToWideChar(CP_ACP, 0, pBuffer, -1, lpwStr, nLen);

	TextBuffer.assign( lpwStr );																								

	delete pBuffer;
	delete lpwStr;
																						
	CloseHandle( hFile );

	if ( ReadStatus == TRUE ) return true;
	
	return false;
}


/////////////////////////////////////////////////////////////////////////////


bool CConfigFile::ReloadFile()
{
	return LoadFile( m_Filename );
}



/////////////////////////////////////////////////////////////////////////////


bool CConfigFile::GetIntValueInRange( wstring Keyword, int& Value, int Min, int Max, int Default )
{
	int Temp;

	if ( GetIntValueFor( Keyword, Temp ) == false ) 
	{
		Value = Default;
		return FALSE;
	}

	if (Temp < Min)
	{
		Value = Min;
		return FALSE;
	}
	
	if (Temp > Max)
	{
		Value = Max;
		return FALSE;
	}

	Value = Temp;

	return true;
}


//--------------------------------------------------------------------------


bool CConfigFile::GetBoolValueFor( wstring Keyword, bool& Value, bool bDefault )
{
	wstring Text;

	Value = bDefault;

	if ( GetStringValueFor( Keyword, Text ) == FALSE ) return false;

	if ( Text == L"" ) return FALSE;


	std::transform(Text.begin(), Text.end(), Text.begin(), toupper);

	if ( Text == L"YES" || Text == L"TRUE" || Text == L"Y" || Text == L"T" )
	{
		Value = true;
		return true;
	}

	Value = ( wcstol( Text.c_str(), 0, 10 ) != 0 );

	return true;
}


/////////////////////////////////////////////////////////////////////////////


bool CConfigFile::GetIntValueFor( wstring Keyword, int& Value )
{
	wstring Text;

	Value = 0;

	if ( GetStringValueFor( Keyword, Text ) == FALSE ) return false;

	if ( Text == L"" ) return false;

	Value = wcstol( Text.c_str(), 0, 10 );

	return true;
}


/////////////////////////////////////////////////////////////////////////////


bool CConfigFile::GetStringValueFor( wstring Keyword, wstring& Value )
{
	int pos;

	transform( Keyword.begin(), Keyword.end(), Keyword.begin(), tolower); 

	wstring Text = TextBuffer;

	TrimLeft( Text );	

	Value = L"";

	//----------------------------------------------
	// loop until end of file or the value is found

	while ( TRUE )
	{
		if ( Text.length() < 1 ) return false;

		//----------------------------------------
		// parse past any leading comment lines

		while ( Text.substr(0,1) == L";" )
		{
			pos = Text.find(L'\n');
			if ( pos == string::npos ) return false;

			Text = Text.substr(pos+1);
			
			TrimLeft( Text );
		}

		//-------------------------------------------

		if ( Text.length() < Keyword.length() ) return false;

		//-------------------------------------------
		// check for the keyword and the following =
		
		wstring FileKeyword = Text.substr( 0, Keyword.length() );

		transform( FileKeyword.begin(), FileKeyword.end(), FileKeyword.begin(), tolower); 

		if ( FileKeyword == Keyword  )
		{
			Text = Text.substr( Keyword.length() );

			TrimLeft( Text );
		
			if ( Text.length() <= 1 ) return false;

			if ( Text.substr(0,1) == L"=" ) 
			{	
				Text = Text.substr(1);

				Value = Text;

				// now get all text until the end of the line
				// or until a comment character is found.
				// this is the value we want for this entry.

				pos = Value.find(L'\n');		

				if ( pos >= 0 ) Value = Value.substr(0,pos);

				pos = Value.find(L';');	

				if ( pos >= 0 ) Value = Value.substr(0,pos);

				// chop off any spaces at start or end of string.

				TrimLeft( Value );
				TrimRight( Value );

				return true;
			}
		}

		//-----------------------------------
		// move onto next line and keep on
		// searching for the keyword.

		pos = Text.find(L'\n');

		if ( pos == string::npos )	return false;	// end of buffer

		Text = Text.substr(pos+1);
		
		TrimLeft( Text );
	}
}


/////////////////////////////////////////////////////////////////////////////

void CConfigFile::TrimLeft( wstring& str )
{
	wstring::iterator i;
	for (i = str.begin(); i != str.end(); i++) 
	{
		if (!isspace(*i)) 
		{
			break;
		}
	}
	if (i == str.end()) 
	{
		str.clear();
	} 
	else 
	{
		str.erase(str.begin(), i);
	}
}


/////////////////////////////////////////////////////////////////////////////


void CConfigFile::TrimRight( wstring& str )
{
	wstring::iterator i;
	
	for (i = str.end() - 1; ;i--) 
	{
		
		if (!isspace(*i)) 
		{
			str.erase(i + 1, str.end());
			break;
		}
		
		if (i == str.begin()) 
		{
			str.clear();
			break;
		}
	}
}


/////////////////////////////////////////////////////////////////////////////



bool CConfigFile::GetHashStringValuesFor( wstring Keyword, stdext::hash_set<wstring>& Values )
{
	Values.clear();

	wstring ValueString;

	BOOL bStatus = GetStringValueFor( Keyword, ValueString );

	if ( bStatus == FALSE ) return false;

	wstring sValue;
	int pos;

	while ( ValueString != L"" )
	{
		pos = ValueString.find(L",");

		if ( pos >= 0 )
		{
			sValue	= ValueString.substr(0,pos);
			ValueString	= ValueString.substr(pos+1);
		}
		else
		{
			sValue	= ValueString;
			ValueString	= L"";
		}

		TrimLeft( sValue );
		TrimRight( sValue );

		Values.insert( sValue );

		// if the last item is a blank then have to add it here.

		if ( pos >= 0 && ValueString == L"") Values.insert(L"");
	}
	
	return true;
}



/////////////////////////////////////////////////////////////////////////////
