//
//		FileEventsMonitor.h
//
//
//		Kernel / User Mode	:	User
//
//		
//		Responsible for 
//		- loading the filter driver
//		- opening a xml file
//		- wait for file events from the driver
//		- filter these and write the ones we want to the xml file.
//
//
// File History
//
//	Date			Person		Comment
//
//  7 Feb 2011		M Lewis		First Written
//
//----------------------------------------------------------------------------

#pragma once
#include <windows.h>
#include <winioctl.h>
#include <hash_set>
#include <shlobj.h>
#include "OOThreadHandler.h"	

#include "fltUser.h"

#include <list>
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <hash_map>
#include <winioctl.h>
#include <tchar.h>

#include "XMLLibWrapper.h"

#include "KernelTypeDefs.h"
#include "FileFilterInterfaceDefs.h"


using namespace std;

typedef pair <wstring, wstring> DosPair;

#define CAPTURE_FILEMON_PORT_NAME			L"\\CaptureFileMonitorPort"

#define FILE_EVENT_FILTER_DRIVER_NAME		L"CaptureFileMonitor"


/* Max Buffer size to allocate */

#define FILE_EVENTS_BUFFER_SIZE		3*65536 

#define FILE_EVENT_WAIT_TIME		50

#define PROCESS_PARENT_CACHE_SIZE	500
#define MAX_PROC_NAME_SIZE			100



class FileEventsMonitor : public Runnable  
{

public:
	FileEventsMonitor( FILE* pLogFile, wchar_t* XmlFilename );

	virtual ~FileEventsMonitor(void);

	void StartMonitorThread();
	void StopMonitorThread();

	void run();

	void SetDeletedFilesDir(  BOOLEAN OnOff, wchar_t* DeletedFilesDir );

	void CopyModifiedFilesToOutputDir( wchar_t WrittenFilesDestDir[] );

	void SetFilterByThreadID( DWORD threadid )
	{
		FilterEventsByProcId = threadid;
	}

	bool bCaptureAllEvents;

	void DumpProcessRelationshipsCache();

private:
//	FILE* XmlFile;
	FILE* LogFile;

	wstring XmlFilename;

	XMLLibWrapper XMLCreation;

	DWORD FilterEventsByProcId;
	
	bool GetEventName(PFILE_EVENT pFileEvent, wstring* fileEventName);
	
	wstring ConvertRawFilePathameToDriveAndPathname(wstring fileObjectName);
	
	void SetupDiskNameToDriveNameMap();
	
	bool isDirectory(wstring filePath);

	wstring ThisAppWorkngDirectory;

	bool LoadFilterDriver();

	BYTE* fileEvents;
	
	OOThreadHandler* fileMonitorThread;

	HANDLE hDriver;
	HANDLE communicationPort;
	HANDLE hMonitorStoppedEvent;

	stdext::hash_map<wstring, wstring> dosNameMap;
	stdext::hash_set<wstring> modifiedFiles;

	bool bIsMonitorRunning;
	bool bIsDriverInstalled;

	void LoadProcessFilterList();

	stdext::hash_set<wstring> ProcessList;

	BOOL CachedGetProcessNameAndParentId(int pid, UINT* pParent, wchar_t** pProcName );

	BOOL GetProcessNameAndParentId(int pid, UINT* pParent, wchar_t* pProcName);

	DWORD	ProcCacheEntries;
	DWORD	ProcId				[ PROCESS_PARENT_CACHE_SIZE ];
	wchar_t ProcName			[ PROCESS_PARENT_CACHE_SIZE ][ MAX_PROC_NAME_SIZE ];
	DWORD	ProcParentId		[ PROCESS_PARENT_CACHE_SIZE ];

};

