//----------------------------------------------------------------------------
//
//		RegEventsMonitor.h
//
//
//		Kernel / User Mode	:	User
//
//		
//		Responsible for 
//		- loading the filter driver
//		- opening a xml file
//		- wait for registry events from the driver
//		- filter these and write the ones we want to the xml file.
//
//
// File History
//
//	Date			Person		Comment
//
//  30 Jan 2010		M Lewis		First Written
//
//----------------------------------------------------------------------------


#pragma once
#include <windows.h>
#include <winioctl.h>
#include "OOThreadHandler.h"
#include <list>
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <hash_map>
#include <hash_set>
#include <winioctl.h>
#include <tchar.h>
#include "XMLLibWrapper.h"


#include "KernelTypeDefs.h"
#include "RegFilterInterfaceDefs.h"


using namespace std;

//---------------------------------------------------

#define REGISTRY_EVENTS_BUFFER_SIZE			5*65536

#define REGISTRY_MONITOR_THREAD_SLEEP_TIME	5

#define PROCESS_PARENT_CACHE_SIZE			500
#define MAX_PROC_NAME_SIZE					100

#define REG_EVENT_FILTER_DRIVER_NAME		L"CaptureRegistryMonitor"
#define REG_EVENT_FILTER_DRIVER_FILENAME	L"CaptureRegistryMonitor.sys"
#define REG_EVENT_FILTER_DRIVER_NAMESPACE	L"\\\\.\\CaptureRegistryMonitor"

//---------------------------------------------------


class RegEventsMonitor : public Runnable 
{
public:

	RegEventsMonitor(  FILE* pLogFile, wchar_t* XmlFilename );
	virtual ~RegEventsMonitor(void);

	void StartMonitorThread();
	void StopMonitorThread();
	
	void SetFilterByThreadID( DWORD threadid )  {  FilterEventsByProcId = threadid;  }

	bool bCaptureAllEvents;

private:

	void run();

//	FILE* XmlFile;
	FILE* LogFile;

	wstring XmlFilename;

	XMLLibWrapper XMLCreation;

	DWORD FilterEventsByProcId;

	wstring GetRegistryEventTypeName(int EventType);

	wstring ConvertPartialRegPathToHiveName( wstring RegistryPath );

//	wstring EncodeXMLChars( wstring in );

	void SetupPartialRegPathToHiveNameMap();

	BOOL GetRegistryCurrentUserKey( wstring* psKey );

	DWORD ExtractRegData( ULONG Type, BYTE* registryData, ULONG dataLengthB, wstring* DataType, wstring* RegData );

	HANDLE hDriverComms;
	HANDLE hThreadStoppedEvent;
	BYTE* RegEventsBuffer;
	
	OOThreadHandler* registryMonitorThread;
	
	UINT	numRegMappings;
	wstring RegPartialPath	[10];
	wstring RegHiveName		[10];

	bool bRegDriverLoaded;
	bool bThreadRunning;

	void LoadProcessFilterList();

	stdext::hash_set<wstring> ProcessList;

	BOOL CachedGetProcessNameAndParentId(int pid, UINT* pParent, wchar_t** pProcName );
	BOOL GetProcessNameAndParentId(int pid, UINT* pParent, wchar_t* pProcName);

	int		ProcCacheEntries;
	DWORD	ProcCacheProcId			[ PROCESS_PARENT_CACHE_SIZE ];
	wchar_t ProcCacheProcName		[ PROCESS_PARENT_CACHE_SIZE ] [ MAX_PROC_NAME_SIZE ];
	DWORD	ProcCacheProcParentId	[ PROCESS_PARENT_CACHE_SIZE ];

	bool LoadFilterDriver();
	bool InstallRegistryKernelDriverAsService();
	bool StartRegistryKernelDriverService();
	void UninstallRegistryKernelDriver();

	SC_HANDLE hService;
};
