//----------------------------------------------------------------------------
//
//	Code to create XML files for Registry and File Events.
//
//  Also includes a number of wrapper functions to tame the COM
//  interface a little within C++.
//  These wrappers are borrowed from a MSDN page
//  ( search for dynamdom )
//
//
//----------------------------------------------------------------------------
//

#include "XMLLibWrapper.h"
#include "comdef.h"


// Macro that calls a COM method returning HRESULT value.
#define CHK_HR(stmt)        do { hr=(stmt); if (FAILED(hr)) goto CleanUp; } while(0)

// Macro to verify memory allcation.
#define CHK_ALLOC(p)        do { if (!(p)) { hr = E_OUTOFMEMORY; goto CleanUp; } } while(0)

// Macro that releases a COM object if not NULL.
#define SAFE_RELEASE(p)     do { if ((p)) { (p)->Release(); (p) = NULL; } } while(0)



XMLLibWrapper::XMLLibWrapper(void)
{
}


XMLLibWrapper::~XMLLibWrapper(void)
{
}


//---------------------------------------------------------


bool XMLLibWrapper::CreateEnvDataDoc()
{
    HRESULT hr = S_OK;
	
		pXMLEnvDataDom = NULL;

		CHK_HR( CreateAndInitDOM(&pXMLEnvDataDom) );

		// Create a processing instruction element.
		CHK_HR( CreateAndAddPINode(pXMLEnvDataDom, L"xml", L"version='1.0'") );

		// Create a comment element.
		CHK_HR( CreateAndAddCommentNode(pXMLEnvDataDom, L"Env Info file created using MSXML DOM object.") );
    
		// Create the root element.
    
		CHK_HR( CreateElement(pXMLEnvDataDom, L"context", &pEnvContextRoot) );

		CHK_HR( CreateAndAddElementNode(pXMLEnvDataDom, L"environmentitems", L"\n", pEnvContextRoot, &pEnvItemsRoot ) );
	
CleanUp:

	if ( FAILED(hr) )
	{
		SAFE_RELEASE( pXMLEnvDataDom );
		SAFE_RELEASE( pEnvContextRoot );
		SAFE_RELEASE( pEnvItemsRoot );
	}

	return !FAILED(hr);
}

//---------------------------------------------------------

bool XMLLibWrapper::WriteEnvDataEntry( PCWSTR Name, PCWSTR Value )
{
	if ( pXMLEnvDataDom == NULL ) return false;

	IXMLDOMElement* pNode;
	
	HRESULT hr = S_OK;
    
	   CHK_HR( CreateAndAddElementNode(pXMLEnvDataDom, L"item", L"\n", pEnvItemsRoot, &pNode ) );
 
	   CHK_HR( CreateAndAddAttributeNode( pXMLEnvDataDom, L"name", Name, pNode) );

	   CHK_HR( CreateAndAddTextNode(pXMLEnvDataDom, Value, pNode ) );
	
CleanUp:

   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}


//---------------------------------------------------------


bool XMLLibWrapper::WriteEnvDataEntry_Sysitem( PCWSTR Name, PCWSTR Value )
{
	if ( pXMLEnvDataDom == NULL ) return false;

	IXMLDOMElement* pNode;
	
	HRESULT hr = S_OK;
    
	   CHK_HR( CreateAndAddElementNode(pXMLEnvDataDom, L"sysitem", L"\n", pEnvItemsRoot, &pNode ) );
 
	   CHK_HR( CreateAndAddAttributeNode( pXMLEnvDataDom, L"name", Name, pNode) );

	   CHK_HR( CreateAndAddTextNode(pXMLEnvDataDom, Value, pNode ) );
	
CleanUp:

   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}

//---------------------------------------------------------


bool XMLLibWrapper::SaveEnvDataDoc( PCWSTR Filename)
{
	if ( pXMLEnvDataDom == NULL ) return false;

	VARIANT varFileName;
    VariantInit(&varFileName);

	HRESULT hr = S_OK;
	
	CHK_HR( CreateAndAddTextNode(pXMLEnvDataDom, L"\n", pEnvItemsRoot) );

    CHK_HR( CreateAndAddTextNode(pXMLEnvDataDom, L"\n", pEnvContextRoot) );
    
	// add <root> to document
    
	CHK_HR( AppendChildToParent(pEnvContextRoot, pXMLEnvDataDom) );

	// get filename into a variant
	
	CHK_HR( VariantFromString( Filename, varFileName ) );

	CHK_HR( pXMLEnvDataDom->save(varFileName) );


CleanUp:

	SAFE_RELEASE( pXMLEnvDataDom );
    SAFE_RELEASE( pEnvItemsRoot );
	SAFE_RELEASE( pEnvContextRoot );

    VariantClear( &varFileName );

	return !FAILED(hr);
}



//---------------------------------------------------------
//

bool XMLLibWrapper::CreateAFileEventsDoc()
{
    HRESULT hr = S_OK;
	
	pXMLFileEventsDom = NULL;

	CHK_HR( CreateAndInitDOM(&pXMLFileEventsDom) );

    // Create a processing instruction element.
    CHK_HR( CreateAndAddPINode(pXMLFileEventsDom, L"xml", L"version='1.0'") );

    // Create a comment element.
    CHK_HR( CreateAndAddCommentNode(pXMLFileEventsDom, L"File Events file created using MSXML DOM object.") );
    
	// Create the root element.
    
	CHK_HR( CreateElement(pXMLFileEventsDom, L"filecapture", &pFileCaptureRoot) );

	CHK_HR( CreateAndAddElementNode(pXMLFileEventsDom, L"fileevents", L"\n", pFileCaptureRoot, &pFileEventsRoot ) );


CleanUp:

	if ( FAILED(hr) )
	{
		SAFE_RELEASE( pXMLFileEventsDom );
		SAFE_RELEASE( pFileCaptureRoot );
		SAFE_RELEASE( pFileEventsRoot );
	}

	return !FAILED(hr);
}


//------------------------------------------------------------

bool XMLLibWrapper::WriteFileDataLostError()
{
	if ( pXMLFileEventsDom == NULL ) return false;

	IXMLDOMElement* pNode  = NULL;
    HRESULT hr = S_OK;
   
    CHK_HR( CreateAndAddElementNode(pXMLFileEventsDom, L"error", L"\n", pFileEventsRoot, &pNode ) );
	CHK_HR( CreateAndAddTextNode(pXMLFileEventsDom, L"Data Lost", pNode ) );

CleanUp:

   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}

//------------------------------------------------------------


bool XMLLibWrapper::WriteFileEvent( PCWSTR procid, PCWSTR process, PCWSTR datetime, PCWSTR operation, PCWSTR Filepath, PCWSTR Filepath2 )
{
	if ( pXMLFileEventsDom == NULL ) return false;

	IXMLDOMElement* pNode = NULL;
	IXMLDOMElement* pNode2 = NULL;
	IXMLDOMElement* pNode3 = NULL;

   HRESULT hr = S_OK;
   
   CHK_HR( CreateAndAddElementNode(pXMLFileEventsDom, L"fileevent", L"\n", pFileEventsRoot, &pNode ) );
 
   CHK_HR( CreateAndAddAttributeNode( pXMLFileEventsDom, L"processid", procid, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLFileEventsDom, L"process", process, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLFileEventsDom, L"datetime", datetime, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLFileEventsDom, L"operation", operation, pNode) );

   CHK_HR( CreateAndAddElementNode(pXMLFileEventsDom, L"path", L"", pNode, &pNode2 ) );
   CHK_HR( CreateAndAddTextNode(pXMLFileEventsDom, Filepath, pNode2 ) );

   if (   wcsnlen( Filepath2, 1 ) != 0  )
   {
		CHK_HR( CreateAndAddElementNode(pXMLFileEventsDom, L"path2", L"", pNode, &pNode3 ) );
		CHK_HR( CreateAndAddTextNode(pXMLFileEventsDom, Filepath2, pNode3 ) );
   }

CleanUp:

   SAFE_RELEASE( pNode3 );
   SAFE_RELEASE( pNode2 );
   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}


//------------------------------------------------------------



bool XMLLibWrapper::SaveFileEventsDoc( PCWSTR Filename)
{
	if ( pXMLFileEventsDom == NULL ) return false;

	VARIANT varFileName;
    VariantInit(&varFileName);

	HRESULT hr = S_OK;
	
	CHK_HR( CreateAndAddTextNode(pXMLFileEventsDom, L"\n", pFileEventsRoot) );

    CHK_HR( CreateAndAddTextNode(pXMLFileEventsDom, L"\n", pFileCaptureRoot) );
    
	// add <root> to document
    
	CHK_HR( AppendChildToParent(pFileCaptureRoot, pXMLFileEventsDom) );

	// get filename into a variant
	
	CHK_HR( VariantFromString( Filename, varFileName ) );

	CHK_HR( pXMLFileEventsDom->save(varFileName) );


CleanUp:

	SAFE_RELEASE( pXMLFileEventsDom );
    SAFE_RELEASE( pFileEventsRoot );
	SAFE_RELEASE( pFileCaptureRoot );

    VariantClear( &varFileName );

	return !FAILED(hr);
}



//---------------------------------------------------------
//

bool XMLLibWrapper::CreateARegEventsDoc()
{
    HRESULT hr = S_OK;
	
	pXMLRegEventsDom = NULL;

	CHK_HR( CreateAndInitDOM(&pXMLRegEventsDom) );

    // Create a processing instruction element.
    CHK_HR( CreateAndAddPINode(pXMLRegEventsDom, L"xml", L"version='1.0'") );

    // Create a comment element.
    CHK_HR( CreateAndAddCommentNode(pXMLRegEventsDom, L"Reg Events file created using MSXML DOM object.") );
    
	// Create the root element.
    
	CHK_HR( CreateElement(pXMLRegEventsDom, L"regcapture", &pRegCaptureRoot) );

	CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"regevents", L"\n", pRegCaptureRoot, &pRegEventsRoot ) );


CleanUp:

	if ( FAILED(hr) )
	{
		SAFE_RELEASE( pXMLRegEventsDom );
		SAFE_RELEASE( pRegCaptureRoot );
		SAFE_RELEASE( pRegEventsRoot );
	}

	return !FAILED(hr);
}


//------------------------------------------------------------

bool XMLLibWrapper::WriteRegDataLostError()
{
	if ( pXMLRegEventsDom == NULL ) return false;

	IXMLDOMElement* pNode  = NULL;
    HRESULT hr = S_OK;
   
    CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"error", L"\n", pRegEventsRoot, &pNode ) );
	CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, L"Data Lost", pNode ) );

CleanUp:

   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}


//------------------------------------------------------------

bool XMLLibWrapper::WriteRegEvent( PCWSTR procid, PCWSTR process, PCWSTR datetime, PCWSTR operation, PCWSTR Regpath, PCWSTR ValueName, PCWSTR DataType, PCWSTR Data )
{
	if ( pXMLRegEventsDom == NULL ) return false;

	IXMLDOMElement* pNode  = NULL;
	IXMLDOMElement* pNode2 = NULL;
	IXMLDOMElement* pNode3 = NULL;

   HRESULT hr = S_OK;
   
   CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"regevent", L"\n", pRegEventsRoot, &pNode ) );
 
   CHK_HR( CreateAndAddAttributeNode( pXMLRegEventsDom, L"processid", procid, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLRegEventsDom, L"process", process, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLRegEventsDom, L"datetime", datetime, pNode) );
   CHK_HR( CreateAndAddAttributeNode( pXMLRegEventsDom, L"operation", operation, pNode) );

   CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"path", L"", pNode, &pNode2 ) );
   CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, Regpath, pNode2 ) );
   CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"name", L"", pNode, &pNode2 ) );
   CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, ValueName, pNode2 ) );

   if (  wcsnlen( DataType, 1 ) != 0 )
   {
		CHK_HR( CreateAndAddElementNode(pXMLRegEventsDom, L"data", L"", pNode, &pNode3 ) );
		CHK_HR( CreateAndAddAttributeNode( pXMLRegEventsDom, L"type", DataType, pNode3 ) );
		CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, Data, pNode3 ) );
   }

CleanUp:

   SAFE_RELEASE( pNode3 );
   SAFE_RELEASE( pNode2 );
   SAFE_RELEASE( pNode );
   
   return !FAILED(hr);
}


//------------------------------------------------------------



bool XMLLibWrapper::SaveRegEventsDoc( PCWSTR Filename)
{
	if ( pXMLRegEventsDom == NULL ) return false;

	VARIANT varFileName;
    VariantInit(&varFileName);

	HRESULT hr = S_OK;
	
	CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, L"\n", pRegEventsRoot) );

    CHK_HR( CreateAndAddTextNode(pXMLRegEventsDom, L"\n", pRegCaptureRoot) );
    
	// add <root> to document
    
	CHK_HR( AppendChildToParent(pRegCaptureRoot, pXMLRegEventsDom) );

	// get filename into a variant
	
	CHK_HR( VariantFromString( Filename, varFileName ) );

	CHK_HR( pXMLRegEventsDom->save(varFileName) );


CleanUp:

	SAFE_RELEASE( pXMLRegEventsDom );
    SAFE_RELEASE( pRegEventsRoot );
	SAFE_RELEASE( pRegCaptureRoot );

    VariantClear( &varFileName );

	return !FAILED(hr);
}



//----------------------------------------------------------------------------------------


// Helper function to create a VT_BSTR variant from a null terminated string. 
HRESULT XMLLibWrapper::VariantFromString(PCWSTR wszValue, VARIANT &Variant)
{
    HRESULT hr = S_OK;
    BSTR bstr = SysAllocString(wszValue);
    CHK_ALLOC(bstr);
    
    V_VT(&Variant)   = VT_BSTR;
    V_BSTR(&Variant) = bstr;

CleanUp:
    return hr;
}

// Helper function to create a DOM instance. 
HRESULT XMLLibWrapper::CreateAndInitDOM(IXMLDOMDocument **ppDoc)
{
    HRESULT hr = CoCreateInstance(__uuidof(DOMDocument60), NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(ppDoc));
    if (SUCCEEDED(hr))
    {
        // these methods should not fail so don't inspect result
        (*ppDoc)->put_async(VARIANT_FALSE);  
        (*ppDoc)->put_validateOnParse(VARIANT_FALSE);
        (*ppDoc)->put_resolveExternals(VARIANT_FALSE);
        (*ppDoc)->put_preserveWhiteSpace(VARIANT_TRUE);
    }
    return hr;
}

// Helper that allocates the BSTR param for the caller.
HRESULT XMLLibWrapper::CreateElement(IXMLDOMDocument *pXMLDom, PCWSTR wszName, IXMLDOMElement **ppElement)
{
    HRESULT hr = S_OK;
    *ppElement = NULL;

    BSTR bstrName = SysAllocString(wszName);
    CHK_ALLOC(bstrName);
    CHK_HR(pXMLDom->createElement(bstrName, ppElement));

CleanUp:
    SysFreeString(bstrName);
    return hr;
}

// Helper function to append a child to a parent node.
HRESULT XMLLibWrapper::AppendChildToParent(IXMLDOMNode *pChild, IXMLDOMNode *pParent)
{
    HRESULT hr = S_OK;
    IXMLDOMNode *pChildOut = NULL;
    CHK_HR(pParent->appendChild(pChild, &pChildOut));

CleanUp:
    SAFE_RELEASE(pChildOut);
    return hr;
}

// Helper function to create and add a processing instruction to a document node.
HRESULT XMLLibWrapper::CreateAndAddPINode(IXMLDOMDocument *pDom, PCWSTR wszTarget, PCWSTR wszData)
{
    HRESULT hr = S_OK;
    IXMLDOMProcessingInstruction *pPI = NULL;

    BSTR bstrTarget = SysAllocString(wszTarget);
    BSTR bstrData = SysAllocString(wszData);
    CHK_ALLOC(bstrTarget && bstrData);
    
    CHK_HR(pDom->createProcessingInstruction(bstrTarget, bstrData, &pPI));
    CHK_HR(AppendChildToParent(pPI, pDom));

CleanUp:
    SAFE_RELEASE(pPI);
    SysFreeString(bstrTarget);
    SysFreeString(bstrData);
    return hr;
}

// Helper function to create and add a comment to a document node.
HRESULT XMLLibWrapper::CreateAndAddCommentNode(IXMLDOMDocument *pDom, PCWSTR wszComment)
{
    HRESULT hr = S_OK;
    IXMLDOMComment *pComment = NULL;
    IXMLDOMText *pText = NULL;

    BSTR bstrComment = SysAllocString(wszComment);
    CHK_ALLOC(bstrComment);
    
    CHK_HR(pDom->createComment(bstrComment, &pComment));
    CHK_HR(AppendChildToParent(pComment, pDom));



CleanUp:
    SAFE_RELEASE(pComment);
    SysFreeString(bstrComment);
    return hr;
}

// Helper function to create and add an attribute to a parent node.
HRESULT XMLLibWrapper::CreateAndAddAttributeNode(IXMLDOMDocument *pDom, PCWSTR wszName, PCWSTR wszValue, IXMLDOMElement *pParent)
{
    HRESULT hr = S_OK;
    IXMLDOMAttribute *pAttribute = NULL;
    IXMLDOMAttribute *pAttributeOut = NULL; // Out param that is not used

    BSTR bstrName = NULL;
    VARIANT varValue;
    VariantInit(&varValue);

    bstrName = SysAllocString(wszName);
    CHK_ALLOC(bstrName);
    CHK_HR(VariantFromString(wszValue, varValue));

    CHK_HR(pDom->createAttribute(bstrName, &pAttribute));
    CHK_HR(pAttribute->put_value(varValue));
    CHK_HR(pParent->setAttributeNode(pAttribute, &pAttributeOut));

CleanUp:
    SAFE_RELEASE(pAttribute);
    SAFE_RELEASE(pAttributeOut);
    SysFreeString(bstrName);
    VariantClear(&varValue);
    return hr;
}

// Helper function to create and append a text node to a parent node.
HRESULT XMLLibWrapper::CreateAndAddTextNode(IXMLDOMDocument *pDom, PCWSTR wszText, IXMLDOMNode *pParent)
{
    HRESULT hr = S_OK;    
    IXMLDOMText *pText = NULL;

    BSTR bstrText = SysAllocString(wszText);
    CHK_ALLOC(bstrText);

    CHK_HR(pDom->createTextNode(bstrText, &pText));
    CHK_HR(AppendChildToParent(pText, pParent));

CleanUp:
    SAFE_RELEASE(pText);
    SysFreeString(bstrText);
    return hr;
}

// Helper function to create and append a CDATA node to a parent node.
HRESULT XMLLibWrapper::CreateAndAddCDATANode(IXMLDOMDocument *pDom, PCWSTR wszCDATA, IXMLDOMNode *pParent)
{
    HRESULT hr = S_OK;
    IXMLDOMCDATASection *pCDATA = NULL;

    BSTR bstrCDATA = SysAllocString(wszCDATA);
    CHK_ALLOC(bstrCDATA);

    CHK_HR(pDom->createCDATASection(bstrCDATA, &pCDATA));
    CHK_HR(AppendChildToParent(pCDATA, pParent));

CleanUp:
    SAFE_RELEASE(pCDATA);
    SysFreeString(bstrCDATA);
    return hr;
}

// Helper function to create and append an element node to a parent node, and pass the newly created
// element node to caller if it wants.
HRESULT XMLLibWrapper::CreateAndAddElementNode(IXMLDOMDocument *pDom, PCWSTR wszName, PCWSTR wszNewline, IXMLDOMNode *pParent, IXMLDOMElement **ppElement = NULL)
{
    HRESULT hr = S_OK;
    IXMLDOMElement* pElement = NULL;

    CHK_HR(CreateElement(pDom, wszName, &pElement));
    // Add NEWLINE+TAB for identation before this element.
    CHK_HR(CreateAndAddTextNode(pDom, wszNewline, pParent));
    // Append this element to parent.
    CHK_HR(AppendChildToParent(pElement, pParent));

CleanUp:
    if (ppElement)
        *ppElement = pElement;  // Caller is repsonsible to release this element.
    else
        SAFE_RELEASE(pElement); // Caller is not interested on this element, so release it.

    return hr;
}


